# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-06-22 16:17
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_vox', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TemplateAttachment',
            fields=[
                ('id', models.AutoField(
                    auto_created=True, primary_key=True,
                    serialize=False, verbose_name='ID')),
                ('key', models.CharField(
                    max_length=500, verbose_name='key')),
                ('template', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    to='django_vox.Template', verbose_name='template')),
            ],
            options={
                'verbose_name': 'template attachment',
            },
        ),
        migrations.AlterField(
            model_name='sitecontact',
            name='address',
            field=models.CharField(
                blank=True, max_length=500, verbose_name='address'),
        ),
    ]
