# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-25 11:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_vox', '0005_activity_table'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='inboxitem',
            name='read',
        ),
        migrations.AddField(
            model_name='inboxitem',
            name='read_at',
            field=models.DateTimeField(
                blank=True, db_index=True, null=True,
                verbose_name='date read'),
        ),
    ]
